function [u, v] = create_ratios_frommatrix(K, truth)
%creates the vectors u and v for every image given a set of kernel matrices
%input: 
%   K is a d x n x n matrix, where d is the number of kernels, and n is
%   the unmber of examples.
%   truth is the ground truth, i.e. cluster membership of points.
%output:
%   u is a d x 1 vector of between-group likeness for each kernel
%   v is a d x 1 vector of within-group likeness for each kernel

classes = unique(truth);
d = size(K,1);
u = zeros(d, 1);
v = zeros(d, 1);

for l=1:d
    KM = squeeze(K(l,:,:));
    for k=1:numel(classes)
        in = truth == classes(k);
        out = truth ~= classes(k);
        v(l) = v(l) + sum(sum(KM(in, in)));
        u(l) = u(l) + sum(sum(KM(in, out)));
    end
end


end