function ALPHA = feature_weigh_convex(U, V)
%Given U and V, solve the feature weighting ratio relaxation problem.
%
%input:
%   U and V are matrices of size n times d where n is the number of training images
%   and d is the number of kernels.
%output:
%   ALPHA is the vector of optimized subkernel weights.

[n d] = size(U);
Q = zeros(d,d);

for g=1:n
    ug = U(g,:)';   %Columns
    vg = V(g,:)';   %Columns
    
    for h = g+1:n
        uh = U(h,:)';   %Columns
        vh = V(h,:)';   %Columns
        
        A = ug * vh' + uh * vg';
        B = vg * vh';

        J = A - B;

        Q = Q + (J + J')/2;
    end
end

%Some normalization.
Q = Q ./ max(max(Q));
for i=1:d, Q(i,i) = 1; end


%Now solve the SDP relaxation
cvx_quiet(true);
cvx_begin
variable ALPHA(d);
variable Z(d,d) symmetric;

minimize trace(Z*Q)
subject to
[Z ALPHA; ALPHA' 1] == semidefinite(d+1);
[Z ALPHA; ALPHA' 1] >= 0;
sum(sum(Z)) >= 1;
sum(ALPHA) == 1;
Z(:,:) >= 0;
ALPHA(:) >= 0;
cvx_end

if(sum(sum(Z - ALPHA*ALPHA')) > 0.05)
    display('Matrices not the same - relaxation may not be tight.');
end

save alphavalues.mat ALPHA

end