function rebuild_KMs(folds, type, features, scales, kernels)
%function rebuild_KMs(folds, type, features, scales, kernels)

sg('loglevel', 'ERROR');
sg('echo', 'OFF');

if nargin < 3 || max(size(features)) == 0
    features = [250000 150000 100000 65000 25000 10000 5000 2000];
end

if nargin < 4 || max(size(kernels)) == 0 || max(size(scales)) == 0
    kernels = 'LPPGG';
    scales = [1 2 3 .5 2];
end

FE = numel(features);
KE = numel(kernels);

%Rows :: examples. Note the transpose later on.
Data = actually_load(sprintf('Data_%s.mat', type), sprintf('Data_%s', type));
truth = actually_load('ground_truth.mat', 'truth');

%Since MCIs are not used for training or testing, the train/test division
%should not include them.
M = numel(find(truth <= 1));

if exist('S.mat', 'file')
    load S.mat S
    iterations = numel(S) / M;
else
    S = zeros(30,M);
    for I=1:30, S(I,:) = randperm(M); end
    save S.mat S
end


PLOT = 0;

if exist(sprintf('kernels_%s', type), 'dir') == 0
    system(sprintf('mkdir kernels_%s', type));
end

tic;
for iter=1:iterations
    s = S(iter,:);
    truth = actually_load('ground_truth.mat', 'truth');

    Test_Train = floor(1:(folds/M):(folds+1));
    Test_Train = Test_Train(s)'; %This gets rid of the unwanted extra element.
    
    for f=1:folds
        train = zeros(numel(truth), 1);
        train(1:M) = Test_Train ~= f;
            
            
        %Create the "good" kernels for classification
        k_file = sprintf('kernels_%s/K_%d_%d.mat', type, iter, f);
        if exist(k_file, 'file') == 0
            disp(sprintf('%d %d', iter, f));
            K = zeros(FE, KE, numel(truth), numel(truth));

            best_file = sprintf('kernels_%s/best_%d_%d.mat', type, iter, f);
            best = get_best(best_file, train, truth, Data);

            for j=1:FE
                D2 = Data(:, best(1:features(j)))';
                for i=1:KE
                    K(j,i,:,:) = get_kernel(D2, kernels(i), scales(i));
                    if PLOT == 1; show_kernel(K(j,i,:,:), get_name(kernels(i), scales(i)), features(j)); end
                end %kernels
                if j==1 , toc, end
            end %Features
            toc

            save(k_file, 'K', 'Test_Train', 'kernels', 'features', 'scales');
        end
    end % Folds

end % Iterations

toc

end % Function


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Get the ranking of features
function best = get_best(best_file, train, truth, Data)

if exist(best_file, 'file') == 0
    AD = truth == -1;
    CN = truth == 1;
    [jnk pGM] = ttest2(Data(AD & train,:), Data(CN & train,:), 10^-8, 'both', 'unequal');
    [jnk best] = sort(pGM);
    save(best_file, 'best');
else
    best = actually_load(best_file, 'best');
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Construct a single kernel
function K = get_kernel(X, kernel, scale)
K=0;

sg('new_svm', 'LIGHT');
sg('clean_features', 'TRAIN');

switch kernel
    case 'G',
        sg('set_kernel', 'GAUSSIAN', 'REAL', 100, scale*size(X,1));
        sg('set_features', 'TRAIN', X);
        sg('init_kernel', 'TRAIN');
        K = sg('get_kernel_matrix', 'TRAIN');
    case 'P',
        sg('set_kernel', 'POLY', 'REAL', 100, scale, false);
        sg('set_features', 'TRAIN', X);
        sg('init_kernel', 'TRAIN');
        K = sg('get_kernel_matrix', 'TRAIN');
    case 'L',
        sg('set_kernel', 'LINEAR', 'REAL', 100, 1);
        sg('set_features', 'TRAIN', X);
        sg('init_kernel', 'TRAIN');
        K = sg('get_kernel_matrix', 'TRAIN');
    case 'H',
        K = hist_kernel(X, scale);
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Display a kernel
function show_kernel(K, type, features)
imagesc(K);
colorbar;
title(sprintf('%s %d', type, features));
pause;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%helper function for debug messages
function name = get_name(kernel, scale)
switch kernel
    case 'G',
        name = sprintf('GAUSSIAN: %f', scale);
    case 'P',
        switch scale
            case 2,
                name = 'QUADRATIC';
            case 3,
                name = 'CUBIC';
            case 4,
                name = 'QUARTIC';
            case 5,
                name = 'QUINTIC';
            case 6,
                name = 'SEXTIC';
            otherwise,
                name = sprintf('%dth order polynomial', scale);
        end
    case 'L',
        name = 'LINEAR';
    case 'H'
        name = sprintf('HISTOGRAM, %d bins', scale);
    otherwise,
        name = 'Unknown kernel type';
end
end
    